/******************************************************************************/
/* Oktalyzer header file.                                                     */
/******************************************************************************/
/* This is part of the APlayer Programming Package (APPP).                    */
/* Copyright (C) 1998-2000 by The APlayer-Team.                               */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may only be used in APlayer related         */
/* software. If you want to use it elsewhere, please contact the author for a */
/* permission.                                                                */
/******************************************************************************/


#ifndef __Oktalyzer_h
#define __Oktalyzer_h

// PolyKit headers
#include "POS.h"
#include "PString.h"
#include "PFile.h"
#include "PSynchronize.h"

// APlayer headers
#include "APlayer.h"


/******************************************************************************/
/* Sample structure                                                           */
/******************************************************************************/
typedef struct Sample
{
	PString		name;
	uint32		length;
	uint16		repeatStart;
	uint16		repeatLength;
	uint16		volume;
	uint16		mode;			// 0 = 8, 1 = 4, 2 = B
	int8 *		sample;
} Sample;



/******************************************************************************/
/* PatternLine structure                                                      */
/******************************************************************************/
typedef struct PatternLine
{
	uint8		note;
	uint8		sampleNum;
	uint8		effect;
	uint8		effectArg;
} PatternLine;



/******************************************************************************/
/* Pattern structure                                                          */
/******************************************************************************/
typedef struct Pattern
{
	int16		lineNum;
	PatternLine *lines;
} Pattern;



/******************************************************************************/
/* ChannelInfo structure                                                      */
/******************************************************************************/
typedef struct ChannelInfo
{
	uint8		currNote;
	int16		currPeriod;
	int8 *		sample;
	uint32		releaseStart;
	uint32		releaseLength;
} ChannelInfo;



/******************************************************************************/
/* Oktalyzer class                                                            */
/******************************************************************************/
class Oktalyzer : public APAddOnPlayer
{
public:
	Oktalyzer(APGlobalData *global);
	virtual ~Oktalyzer(void);

	float GetVersion(void);
	uint32 GetSupportFlags(void);

	PString GetName(uint32 number);
	PString GetDescription(uint32 number);
	PString GetModTypeString(uint32 index);

	ap_result ModuleCheck(uint32 index, PFile *file);
	ap_result LoadModule(uint32 index, PFile *file);

	bool InitPlayer(void);
	void EndPlayer(void);
	void InitSound(uint16 songNum);
	void Play(void);

	uint16 GetVirtualChannels(void);

	int16 GetSongLength(void);
	int16 GetSongPosition(void);
	void SetSongPosition(int16 pos);

	uint32 GetInfoCount(void);
	PString GetInfoString(uint32 line, ap_infoType type);

	uint32 GetSampleCount(void);
	void GetSampleInfo(uint32 num, APSampleInfo *info);

protected:
	void ParseCMOD(uint32 chunkSize, PFile *file);
	void ParseSAMP(uint32 chunkSize, PFile *file);
	void ParseSPEE(uint32 chunkSize, PFile *file);
	void ParseSLEN(uint32 chunkSize, PFile *file);
	void ParsePLEN(uint32 chunkSize, PFile *file);
	void ParsePATT(uint32 chunkSize, PFile *file);
	void ParsePBOD(uint32 chunkSize, PFile *file);
	void ParseSBOD(uint32 chunkSize, PFile *file);

	void ShowError(uint32 id);
	void ShowError(PString errorStr);
	void Cleanup(void);

	void FindNextPatternLine(void);
	void PlayPatternLine(void);
	void PlayChannel(uint32 chanNum);
	void SetVolumes(void);
	void DoEffects(void);
	void DoChannelEffect(uint32 chanNum);
	void PlayNote(uint32 chanNum, ChannelInfo *chanData, int8 note);

	uint32 strings;
	PMutex *readyToPlay;

	uint32 sampNum;
	uint16 pattNum;
	uint16 songLength;
	uint16 chanNum;
	uint16 startSpeed;

	uint16 currentSpeed;
	uint16 speedCounter;
	int16 songPos;
	int16 newSongPos;
	int16 pattPos;
	bool filterStatus;

	bool channelFlags[4];
	uint8 patternTable[128];
	Sample *sampleInfo;
	Pattern **patterns;

	PatternLine currLine[8];
	ChannelInfo chanInfo[8];
	int8 chanVol[8];
	uint8 chanIndex[8];

	uint32 readPatt;
	uint32 readSamp;
	uint32 realUsedSampNum;
};

#endif
